@@ -1,4 +1,4 @@
-/*      $Id: xfwm4_shortcuteditor.c,v 1.32 2004/11/11 15:59:00 olivier Exp $
+/*      $Id: xfwm4_shortcuteditor.c,v 1.38.2.2 2005/01/14 11:41:37 pollux Exp $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -60,7 +60,6 @@ cb_popup_del_menu (GtkWidget *widget, gpointer data)
         ThemeInfo *ti;
 
         gchar *theme_name = NULL;
-        gchar *theme_file = NULL;
 
         selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (itf->treeview2));
 
@@ -69,35 +68,41 @@ cb_popup_del_menu (GtkWidget *widget, gpointer data)
 
         ti = find_theme_info_by_name (theme_name, keybinding_theme_list);
 
-        theme_file = g_build_filename (ti->path, KEY_SUFFIX, KEYTHEMERC, NULL);
-        
-        if (unlink (theme_file) == 0)
-        {
-            /* refresh list */
-          while (keybinding_theme_list)
-          {
-              theme_info_free ((ThemeInfo *)keybinding_theme_list->data);
-              keybinding_theme_list = g_list_next (keybinding_theme_list);
-          }
-          g_list_free (keybinding_theme_list);
-        
-          g_free (current_key_theme);
-          current_key_theme = g_strdup ("Default");
-          keybinding_theme_list = NULL;
-          keybinding_theme_list = read_themes (keybinding_theme_list, itf->treeview2, itf->scrolledwindow2,
-                                               KEYBINDING_THEMES, current_key_theme);
-          gtk_widget_set_sensitive (itf->treeview3, FALSE);
-          gtk_widget_set_sensitive (itf->treeview4, FALSE);
-          loadtheme_in_treeview (find_theme_info_by_name ("Default", keybinding_theme_list), itf);
-        
-          /* tell it to the mcs manager */
-          mcs_manager_set_string (itf->mcs_plugin->manager, "Xfwm/KeyThemeName", CHANNEL2, current_key_theme);
-          mcs_manager_notify (itf->mcs_plugin->manager, CHANNEL2);
-          write_options (itf->mcs_plugin);
-        }
+	if (ti)
+	{
+	    gchar *theme_file = NULL;
 
+	    theme_file = g_build_filename (ti->path, KEY_SUFFIX, KEYTHEMERC, NULL);
+	    if (unlink (theme_file) != 0)
+	      g_warning ("Unable to remove the theme file !");
+	    g_free (theme_file);
+	}
+	else
+	    g_warning ("Cannot find the keytheme !");
+
+	/* refresh list */
+	while (keybinding_theme_list)
+	{
+	    theme_info_free ((ThemeInfo *)keybinding_theme_list->data);
+	    keybinding_theme_list = g_list_next (keybinding_theme_list);
+	}
+	g_list_free (keybinding_theme_list);
+	
+	g_free (current_key_theme);
+	current_key_theme = g_strdup ("Default");
+	keybinding_theme_list = NULL;
+	keybinding_theme_list = read_themes (keybinding_theme_list, itf->treeview2, itf->scrolledwindow2,
+					     KEYBINDING_THEMES, current_key_theme);
+	gtk_widget_set_sensitive (itf->treeview3, FALSE);
+	gtk_widget_set_sensitive (itf->treeview4, FALSE);
+	loadtheme_in_treeview (find_theme_info_by_name ("Default", keybinding_theme_list), itf);
+	
+	/* tell it to the mcs manager */
+	mcs_manager_set_string (itf->mcs_plugin->manager, "Xfwm/KeyThemeName", CHANNEL2, current_key_theme);
+	mcs_manager_notify (itf->mcs_plugin->manager, CHANNEL2);
+	write_options (itf->mcs_plugin);
+    
         g_free (theme_name);
-        g_free (theme_file);
     }
 }
 
@@ -117,7 +122,7 @@ cb_popup_add_menu (GtkWidget *widget, gpointer data)
     gchar *new_theme_file = NULL;
 
     itf = (Itf *) data;
-    
+
     dialog = gtk_dialog_new_with_buttons (_("Add keybinding theme"), GTK_WINDOW (itf->xfwm4_dialog),
                                           GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                                           GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, 
@@ -234,6 +239,7 @@ cb_popup_menu (GtkTreeView *treeview, GdkEventButton *event, gpointer data)
     if ((event->button == 3) && (event->type == GDK_BUTTON_PRESS))
     {
         GtkTreePath *path;
+	GdkScreen *screen;
 
         if (gtk_tree_view_get_path_at_pos (treeview, event->x, event->y, &path, NULL, NULL, NULL))
         {
@@ -252,10 +258,15 @@ cb_popup_menu (GtkTreeView *treeview, GdkEventButton *event, gpointer data)
             
             ti = find_theme_info_by_name (theme_name, keybinding_theme_list);
 
-            gtk_tree_selection_unselect_all (selection);
-            gtk_tree_selection_select_path (selection, path);
-
-            gtk_widget_set_sensitive (itf->popup_del_menuitem, ti->user_writable);
+	    if (ti)
+	    {
+	        gtk_tree_selection_unselect_all (selection);
+		gtk_tree_selection_select_path (selection, path);
+	      
+		gtk_widget_set_sensitive (itf->popup_del_menuitem, ti->user_writable);
+	    }
+	    else
+	        g_warning ("Cannot find the keytheme !");
 
             g_free (theme_name);
         }
@@ -264,8 +275,11 @@ cb_popup_menu (GtkTreeView *treeview, GdkEventButton *event, gpointer data)
             gtk_widget_set_sensitive (itf->popup_del_menuitem, FALSE);
         }
         
+       
+	screen = xfce_gdk_display_locate_monitor_with_pointer (NULL, NULL);
+	gtk_menu_set_screen (GTK_MENU (itf->popup_menu), screen ? screen : gdk_screen_get_default ());
         gtk_menu_popup (GTK_MENU (itf->popup_menu), NULL, NULL, NULL, NULL,
-                      event->button, gtk_get_current_event_time());
+			event->button, gtk_get_current_event_time());
         return TRUE;
     }
 
@@ -305,6 +319,15 @@ loadtheme_in_treeview (ThemeInfo *ti, gpointer data)
     {
         g_free (user_theme_file);
         user_theme_file = g_strdup (default_theme_file);
+	gtk_widget_set_sensitive (itf->treeview3, FALSE);
+	gtk_widget_set_sensitive (itf->treeview4, FALSE);
+	gtk_widget_set_sensitive (itf->del_button, FALSE);
+    }
+    else
+    {
+      	gtk_widget_set_sensitive (itf->treeview3, TRUE);
+	gtk_widget_set_sensitive (itf->treeview4, TRUE);
+	gtk_widget_set_sensitive (itf->del_button, TRUE);
     }
 
     default_rc = xfce_rc_simple_open (default_theme_file, TRUE);
@@ -334,257 +357,293 @@ loadtheme_in_treeview (ThemeInfo *ti, gpointer data)
         if (g_ascii_strcasecmp (*shortcut, "close_window_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Close window"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Close window"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "maximize_window_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Maximize window"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Maximize window"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "maximize_vert_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Maximize window vertically"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Maximize window vertically"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "maximize_horiz_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Maximize window horizontally"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Maximize window horizontally"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "hide_window_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Hide window"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Hide window"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "shade_window_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Shade window"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Shade window"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "stick_window_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Stick window"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Stick window"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "cycle_windows_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Cycle windows"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Cycle windows"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "move_window_up_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Move window up"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Move window up"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "move_window_down_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Move window down"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Move window down"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "move_window_left_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Move window left"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Move window left"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "move_window_right_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Move window right"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Move window right"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "resize_window_up_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Resize window up"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Resize window up"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "resize_window_down_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Resize window down"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Resize window down"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "resize_window_left_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Resize window left"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Resize window left"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "resize_window_right_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Resize window right"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Resize window right"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "raise_window_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Raise window"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Raise window"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "lower_window_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Lower window"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Lower window"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "fullscreen_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Toggle fullscreen"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Toggle fullscreen"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "up_workspace_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Upper workspace"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Upper workspace"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "down_workspace_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Bottom workspace"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Bottom workspace"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "left_workspace_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Left workspace"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Left workspace"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "right_workspace_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Right workspace"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Right workspace"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "next_workspace_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Next workspace"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Next workspace"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "prev_workspace_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Previous workspace"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Previous workspace"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "add_workspace_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Add workspace"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Add workspace"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "del_workspace_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Delete workspace"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Delete workspace"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "workspace_1_key") == 0)
         {
+	    gchar *text;
+	    
+	    text = g_strdup_printf (_("Workspace %d"), 1);
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Workspace 1"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, text, COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
+	    g_free (text);
         }
         else if (g_ascii_strcasecmp (*shortcut, "workspace_2_key") == 0)
         {
+	    gchar *text;
+	    
+	    text = g_strdup_printf (_("Workspace %d"), 2);
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Workspace 2"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, text, COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
+	    g_free (text);
         }
         else if (g_ascii_strcasecmp (*shortcut, "workspace_3_key") == 0)
         {
+	    gchar *text;
+	    
+	    text = g_strdup_printf (_("Workspace %d"), 3);
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Workspace 3"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, text, COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
+	    g_free (text);
         }
         else if (g_ascii_strcasecmp (*shortcut, "workspace_4_key") == 0)
         {
+	    gchar *text;
+	    
+	    text = g_strdup_printf (_("Workspace %d"), 4);
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Workspace 4"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, text, COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
+	    g_free (text);
         }
         else if (g_ascii_strcasecmp (*shortcut, "workspace_5_key") == 0)
         {
+	    gchar *text;
+	    
+	    text = g_strdup_printf (_("Workspace %d"), 5);
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Workspace 5"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, text, COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
+	    g_free (text);
         }
         else if (g_ascii_strcasecmp (*shortcut, "workspace_6_key") == 0)
         {
+	    gchar *text;
+	    
+	    text = g_strdup_printf (_("Workspace %d"), 6);
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Workspace 6"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, text, COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
+	    g_free (text);
         }
         else if (g_ascii_strcasecmp (*shortcut, "workspace_7_key") == 0)
         {
+	    gchar *text;
+	    
+	    text = g_strdup_printf (_("Workspace %d"), 7);
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Workspace 7"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, text, COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
+	    g_free (text);
         }
         else if (g_ascii_strcasecmp (*shortcut, "workspace_8_key") == 0)
         {
+	    gchar *text;
+	    
+	    text = g_strdup_printf (_("Workspace %d"), 8);
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Workspace 8"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, text, COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
+	    g_free (text);
         }
         else if (g_ascii_strcasecmp (*shortcut, "workspace_9_key") == 0)
         {
+	    gchar *text;
+	    
+	    text = g_strdup_printf (_("Workspace %d"), 9);
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Workspace 9"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, text, COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
+	    g_free (text);
         }
         else if (g_ascii_strcasecmp (*shortcut, "move_window_next_workspace_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Move window to next workspace"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Move window to next workspace"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "move_window_prev_workspace_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Move window to previous workspace"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Move window to previous workspace"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "move_window_up_workspace_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Move window to upper workspace"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Move window to upper workspace"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "move_window_down_workspace_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Move window to bottom workspace"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Move window to bottom workspace"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "move_window_left_workspace_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Move window to left workspace"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Move window to left workspace"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "move_window_right_workspace_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Move window to right workspace"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Move window to right workspace"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "move_window_workspace_1_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Move window to workspace 1"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Move window to workspace 1"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "move_window_workspace_2_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Move window to workspace 2"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Move window to workspace 2"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "move_window_workspace_3_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Move window to workspace 3"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Move window to workspace 3"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "move_window_workspace_4_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Move window to workspace 4"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Move window to workspace 4"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "move_window_workspace_5_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Move window to workspace 5"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Move window to workspace 5"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "move_window_workspace_6_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Move window to workspace 6"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Move window to workspace 6"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "move_window_workspace_7_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Move window to workspace 7"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Move window to workspace 7"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "move_window_workspace_8_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Move window to workspace 8"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Move window to workspace 8"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "move_window_workspace_9_key") == 0)
         {
             gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Move window to workspace 9"), COLUMN_SHORTCUT, entry_value, -1);
+            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, _("Move window to workspace 9"), COLUMN_SHORTCUT, entry_value, COLUMN_NAME, *shortcut, -1);
         }
         else if (g_ascii_strcasecmp (*shortcut, "shortcut_1_key") == 0)
         {
@@ -737,11 +796,10 @@ savetree3_foreach_func (GtkTreeModel * model, GtkTreePath * path, GtkTreeIter *
 {
     FILE *file = (FILE *) data;
     gchar *line = NULL;
-    gchar *command = NULL;
-    gchar xfwm4_command[50] = "";
+    gchar *name = NULL;
     gchar *shortcut = NULL;
 
-    gtk_tree_model_get (model, iter, COLUMN_COMMAND, &command, COLUMN_SHORTCUT, &shortcut, -1);
+    gtk_tree_model_get (model, iter, COLUMN_NAME, &name, COLUMN_SHORTCUT, &shortcut, -1);
 
     if (strlen (shortcut) == 0)
     {
@@ -749,188 +807,11 @@ savetree3_foreach_func (GtkTreeModel * model, GtkTreePath * path, GtkTreeIter *
         shortcut = g_strdup ("none");
     }
 
-    if (strcmp (command, _("Close window")) == 0)
-    {
-        strcpy (xfwm4_command, "close_window_key");
-    }
-    else if (strcmp (command, _("Maximize window")) == 0)
-    {
-        strcpy (xfwm4_command, "maximize_window_key");
-    }
-    else if (strcmp (command, _("Maximize window vertically")) == 0)
-    {
-        strcpy (xfwm4_command, "maximize_vert_key");
-    }
-    else if (strcmp (command, _("Maximize window horizontally")) == 0)
-    {
-        strcpy (xfwm4_command, "maximize_horiz_key");
-    }
-    else if (strcmp (command, _("Hide window")) == 0)
-    {
-        strcpy (xfwm4_command, "hide_window_key");
-    }
-    else if (strcmp (command, _("Shade window")) == 0)
-    {
-        strcpy (xfwm4_command, "shade_window_key");
-    }
-    else if (strcmp (command, _("Stick window")) == 0)
-    {
-        strcpy (xfwm4_command, "stick_window_key");
-    }
-    else if (strcmp (command, _("Cycle windows")) == 0)
-    {
-        strcpy (xfwm4_command, "cycle_windows_key");
-    }
-    else if (strcmp (command, _("Move window up")) == 0)
-    {
-        strcpy (xfwm4_command, "move_window_up_key");
-    }
-    else if (strcmp (command, _("Move window down")) == 0)
-    {
-        strcpy (xfwm4_command, "move_window_down_key");
-    }
-    else if (strcmp (command, _("Move window left")) == 0)
-    {
-        strcpy (xfwm4_command, "move_window_left_key");
-    }
-    else if (strcmp (command, _("Move window right")) == 0)
-    {
-        strcpy (xfwm4_command, "move_window_right_key");
-    }
-    else if (strcmp (command, _("Resize window up")) == 0)
-    {
-        strcpy (xfwm4_command, "resize_window_up_key");
-    }
-    else if (strcmp (command, _("Resize window down")) == 0)
-    {
-        strcpy (xfwm4_command, "resize_window_down_key");
-    }
-    else if (strcmp (command, _("Resize window left")) == 0)
-    {
-        strcpy (xfwm4_command, "resize_window_left_key");
-    }
-    else if (strcmp (command, _("Resize window right")) == 0)
-    {
-        strcpy (xfwm4_command, "resize_window_right_key");
-    }
-    else if (strcmp (command, _("Raise window")) == 0)
-    {
-        strcpy (xfwm4_command, "raise_window_key");
-    }
-    else if (strcmp (command, _("Lower window")) == 0)
-    {
-        strcpy (xfwm4_command, "lower_window_key");
-    }
-    else if (strcmp (command, _("Toggle fullscreen")) == 0)
-    {
-        strcpy (xfwm4_command, "fullscreen_key");
-    }
-    else if (strcmp (command, _("Next workspace")) == 0)
-    {
-        strcpy (xfwm4_command, "next_workspace_key");
-    }
-    else if (strcmp (command, _("Previous workspace")) == 0)
-    {
-        strcpy (xfwm4_command, "prev_workspace_key");
-    }
-    else if (strcmp (command, _("Add workspace")) == 0)
-    {
-        strcpy (xfwm4_command, "add_workspace_key");
-    }
-    else if (strcmp (command, _("Delete workspace")) == 0)
-    {
-        strcpy (xfwm4_command, "del_workspace_key");
-    }
-    else if (strcmp (command, _("Workspace 1")) == 0)
-    {
-        strcpy (xfwm4_command, "workspace_1_key");
-    }
-    else if (strcmp (command, _("Workspace 2")) == 0)
-    {
-        strcpy (xfwm4_command, "workspace_2_key");
-    }
-    else if (strcmp (command, _("Workspace 3")) == 0)
-    {
-        strcpy (xfwm4_command, "workspace_3_key");
-    }
-    else if (strcmp (command, _("Workspace 4")) == 0)
-    {
-        strcpy (xfwm4_command, "workspace_4_key");
-    }
-    else if (strcmp (command, _("Workspace 5")) == 0)
-    {
-        strcpy (xfwm4_command, "workspace_5_key");
-    }
-    else if (strcmp (command, _("Workspace 6")) == 0)
-    {
-        strcpy (xfwm4_command, "workspace_6_key");
-    }
-    else if (strcmp (command, _("Workspace 7")) == 0)
-    {
-        strcpy (xfwm4_command, "workspace_7_key");
-    }
-    else if (strcmp (command, _("Workspace 8")) == 0)
-    {
-        strcpy (xfwm4_command, "workspace_8_key");
-    }
-    else if (strcmp (command, _("Workspace 9")) == 0)
-    {
-        strcpy (xfwm4_command, "workspace_9_key");
-    }
-    else if (strcmp (command, _("Move window to next workspace")) == 0)
-    {
-        strcpy (xfwm4_command, "move_window_next_workspace_key");
-    }
-    else if (strcmp (command, _("Move window to previous workspace")) == 0)
-    {
-        strcpy (xfwm4_command, "move_window_prev_workspace_key");
-    }
-    else if (strcmp (command, _("Move window to workspace 1")) == 0)
-    {
-        strcpy (xfwm4_command, "move_window_workspace_1_key");
-    }
-    else if (strcmp (command, _("Move window to workspace 2")) == 0)
-    {
-        strcpy (xfwm4_command, "move_window_workspace_2_key");
-    }
-    else if (strcmp (command, _("Move window to workspace 3")) == 0)
-    {
-        strcpy (xfwm4_command, "move_window_workspace_3_key");
-    }
-    else if (strcmp (command, _("Move window to workspace 4")) == 0)
-    {
-        strcpy (xfwm4_command, "move_window_workspace_4_key");
-    }
-    else if (strcmp (command, _("Move window to workspace 5")) == 0)
-    {
-        strcpy (xfwm4_command, "move_window_workspace_5_key");
-    }
-    else if (strcmp (command, _("Move window to workspace 6")) == 0)
-    {
-        strcpy (xfwm4_command, "move_window_workspace_6_key");
-    }
-    else if (strcmp (command, _("Move window to workspace 7")) == 0)
-    {
-        strcpy (xfwm4_command, "move_window_workspace_7_key");
-    }
-    else if (strcmp (command, _("Move window to workspace 8")) == 0)
-    {
-        strcpy (xfwm4_command, "move_window_workspace_8_key");
-    }
-    else if (strcmp (command, _("Move window to workspace 9")) == 0)
-    {
-        strcpy (xfwm4_command, "move_window_workspace_9_key");
-    }
-    else 
-    {
-        strcpy (xfwm4_command, command);
-    }
-
-    line = g_strdup_printf ("%s=%s\n", xfwm4_command, shortcut);
+    line = g_strdup_printf ("%s=%s\n", name, shortcut);
 
     fputs (line, file);
 
-    g_free (command);
+    g_free (name);
     g_free (shortcut);
     g_free (line);
 
