@@ -1,4 +1,4 @@
-/*      $Id: focus.c,v 1.41 2004/12/14 17:45:26 olivier Exp $
+/*      $Id: focus.c,v 1.43.2.4 2005/02/19 14:13:30 olivier Exp $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -83,7 +83,7 @@ clientGetTopMostFocusable (ScreenInfo *screen_info, int layer, Client * exclude)
         {
             if ((c->win_layer <= layer) && FLAG_TEST (c->xfwm_flags, XFWM_FLAG_VISIBLE))
             {
-                if (clientSelectMask (c, 0, WINDOW_NORMAL | WINDOW_DIALOG | WINDOW_MODAL_DIALOG))
+                if (clientSelectMask (c, 0, WINDOW_REGULAR_FOCUSABLE))
                 {
                     top_client.prefered = c;
                 }
@@ -109,11 +109,15 @@ clientFocusTop (ScreenInfo *screen_info, int layer)
     top_client = clientGetTopMostFocusable (screen_info, layer, NULL);
     if (top_client.prefered)
     {
-        clientSetFocus (screen_info, top_client.prefered, CurrentTime, NO_FOCUS_FLAG);
+        clientSetFocus (screen_info, top_client.prefered, 
+                        myDisplayGetCurrentTime (screen_info->display_info), 
+                        NO_FOCUS_FLAG);
     }
     else
     {
-        clientSetFocus (screen_info, top_client.highest, CurrentTime, NO_FOCUS_FLAG);
+        clientSetFocus (screen_info, top_client.highest, 
+                        myDisplayGetCurrentTime (screen_info->display_info), 
+                        NO_FOCUS_FLAG);
     }
 }
 
@@ -150,6 +154,7 @@ clientFocusNew(Client * c)
         }
     }
 
+    clientGrabMouseButton (c);
     if (give_focus || FLAG_TEST(c->flags, CLIENT_FLAG_STATE_MODAL))
     {
         if (client_focus)
@@ -159,7 +164,9 @@ clientFocusNew(Client * c)
                 clientRaise (c);
             }
         }
-        clientSetFocus (c->screen_info, c, CurrentTime, FOCUS_IGNORE_MODAL);
+        clientSetFocus (screen_info, c, 
+                        myDisplayGetCurrentTime (display_info), 
+                        FOCUS_IGNORE_MODAL);
     }
     else
     {
@@ -219,7 +226,7 @@ clientGetNext (Client * c, int mask)
         for (c2 = c->next, i = 0; (c2) && (i < screen_info->client_count - 1);
             c2 = c2->next, i++)
         {
-            if (clientSelectMask (c2, mask, WINDOW_NORMAL | WINDOW_DIALOG | WINDOW_MODAL_DIALOG))
+            if (clientSelectMask (c2, mask, WINDOW_REGULAR_FOCUSABLE))
             {
                 return c2;
             }
@@ -242,7 +249,7 @@ clientGetPrevious (Client * c, int mask)
         for (c2 = c->prev, i = 0; (c2) && (i < screen_info->client_count);
             c2 = c2->prev, i++)
         {
-            if (clientSelectMask (c2, mask, WINDOW_NORMAL | WINDOW_DIALOG | WINDOW_MODAL_DIALOG))
+            if (clientSelectMask (c2, mask, WINDOW_REGULAR_FOCUSABLE))
             {
                 return c2;
             }
@@ -319,7 +326,9 @@ clientPassFocus (ScreenInfo *screen_info, Client *c, Client *exclude)
     {
         new_focus = top_most.prefered ? top_most.prefered : top_most.highest;
     }
-    clientSetFocus (screen_info, new_focus, CurrentTime, FOCUS_IGNORE_MODAL | FOCUS_FORCE);
+    clientSetFocus (screen_info, new_focus, 
+                    myDisplayGetCurrentTime (screen_info->display_info), 
+                    FOCUS_IGNORE_MODAL | FOCUS_FORCE);
 }
 
 gboolean
@@ -418,7 +427,7 @@ clientUpdateFocus (ScreenInfo *screen_info, Client * c, unsigned short flags)
         if (c)
         {
             clientAdjustFullscreenLayer (c2, FALSE);
-            clientRaise (c);
+            /* clientRaise (c); */
         }
         frameDraw (c2, FALSE, FALSE);
     }
