@@ -1,4 +1,4 @@
-/*      $Id: frame.c,v 1.75 2004/11/03 15:15:02 olivier Exp $
+/*      $Id: frame.c,v 1.76.2.3 2005/01/28 22:19:12 olivier Exp $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -568,164 +568,190 @@ frameSetShape (Client * c, int state, ClientPixmapCache * pm_cache, int button_x
         return;
     }
 
-    temp = XCreateSimpleWindow (display_info->dpy, screen_info->xroot, 0, 0, frameWidth (c), frameHeight (c), 0, 0, 0);
+    temp = XCreateSimpleWindow (display_info->dpy, c->frame, 0, 0, frameWidth (c), frameHeight (c), 0, 0, 0);
 
     if (FLAG_TEST (c->flags, CLIENT_FLAG_SHADED))
     {
         rect.x = 0;
         rect.y = 0;
-        rect.width = frameWidth (c);
+        rect.width  = frameWidth (c);
         rect.height = frameHeight (c);
         XShapeCombineRectangles (display_info->dpy, temp, ShapeBounding, 0, 0, &rect, 1,
-            ShapeSubtract, 0);
+                                 ShapeSubtract, 0);
     }
     else
     {
         XShapeCombineShape (display_info->dpy, temp, ShapeBounding, frameLeft (c),
-            frameTop (c), c->window, ShapeBounding, ShapeSet);
+                            frameTop (c), c->window, ShapeBounding, ShapeSet);
     }
     if (pm_cache)
     {
-        XShapeCombineMask (display_info->dpy, MYWINDOW_XWINDOW (c->title), ShapeBounding, 0,
-            0, pm_cache->pm_title[state].mask, ShapeSet);
+        XShapeCombineMask (display_info->dpy, MYWINDOW_XWINDOW (c->title), ShapeBounding,
+                           0, 0, pm_cache->pm_title[state].mask, ShapeSet);
         if (!FLAG_TEST (c->flags, CLIENT_FLAG_SHADED))
         {
-            XShapeCombineMask (display_info->dpy, MYWINDOW_XWINDOW (c->sides[SIDE_LEFT]),
-                ShapeBounding, 0, 0,
-                pm_cache->pm_sides[SIDE_LEFT][state].mask, ShapeSet);
-            XShapeCombineMask (display_info->dpy, MYWINDOW_XWINDOW (c->sides[SIDE_RIGHT]),
-                ShapeBounding, 0, 0,
-                pm_cache->pm_sides[SIDE_RIGHT][state].mask, ShapeSet);
+            if (xfwmWindowVisible (&c->sides[SIDE_LEFT]))
+            {
+                XShapeCombineMask (display_info->dpy, MYWINDOW_XWINDOW (c->sides[SIDE_LEFT]),
+                                   ShapeBounding, 0, 0, pm_cache->pm_sides[SIDE_LEFT][state].mask, ShapeSet);
+            }
+
+            if (xfwmWindowVisible (&c->sides[SIDE_RIGHT]))
+            {
+                XShapeCombineMask (display_info->dpy, MYWINDOW_XWINDOW (c->sides[SIDE_RIGHT]),
+                                   ShapeBounding, 0, 0, pm_cache->pm_sides[SIDE_RIGHT][state].mask, ShapeSet);
+            }
+        }
+
+        if (xfwmWindowVisible (&c->sides[SIDE_BOTTOM]))
+        {
+            XShapeCombineMask (display_info->dpy, MYWINDOW_XWINDOW (c->sides[SIDE_BOTTOM]),
+                               ShapeBounding, 0, 0, pm_cache->pm_sides[SIDE_BOTTOM][state].mask, ShapeSet);
+        }
+
+        if (xfwmWindowVisible (&c->corners[CORNER_BOTTOM_LEFT]))
+        {
+            XShapeCombineMask (display_info->dpy, MYWINDOW_XWINDOW (c->corners[CORNER_BOTTOM_LEFT]), 
+                               ShapeBounding, 0, 0, screen_info->corners[CORNER_BOTTOM_LEFT][state].mask, ShapeSet);
+        }
+
+        if (xfwmWindowVisible (&c->corners[CORNER_BOTTOM_RIGHT]))
+        {
+            XShapeCombineMask (display_info->dpy, MYWINDOW_XWINDOW (c->corners[CORNER_BOTTOM_RIGHT]),
+                               ShapeBounding, 0, 0, screen_info->corners[CORNER_BOTTOM_RIGHT][state].mask, ShapeSet);
+        }
+
+        if (xfwmWindowVisible (&c->corners[CORNER_TOP_LEFT]))
+        {
+            XShapeCombineMask (display_info->dpy, MYWINDOW_XWINDOW (c->corners[CORNER_TOP_LEFT]), 
+                               ShapeBounding, 0, 0, screen_info->corners[CORNER_TOP_LEFT][state].mask, ShapeSet);
+        }
+
+        if (xfwmWindowVisible (&c->corners[CORNER_TOP_RIGHT]))
+        {
+            XShapeCombineMask (display_info->dpy, MYWINDOW_XWINDOW (c->corners[CORNER_TOP_RIGHT]), 
+                               ShapeBounding, 0, 0, screen_info->corners[CORNER_TOP_RIGHT][state].mask, ShapeSet);
         }
-        XShapeCombineMask (display_info->dpy, MYWINDOW_XWINDOW (c->sides[SIDE_BOTTOM]),
-            ShapeBounding, 0, 0, pm_cache->pm_sides[SIDE_BOTTOM][state].mask,
-            ShapeSet);
-        XShapeCombineMask (display_info->dpy,
-            MYWINDOW_XWINDOW (c->corners[CORNER_BOTTOM_LEFT]), ShapeBounding,
-            0, 0, screen_info->corners[CORNER_BOTTOM_LEFT][state].mask, ShapeSet);
-        XShapeCombineMask (display_info->dpy,
-            MYWINDOW_XWINDOW (c->corners[CORNER_BOTTOM_RIGHT]), ShapeBounding,
-            0, 0, screen_info->corners[CORNER_BOTTOM_RIGHT][state].mask, ShapeSet);
-        XShapeCombineMask (display_info->dpy,
-            MYWINDOW_XWINDOW (c->corners[CORNER_TOP_LEFT]), ShapeBounding, 0,
-            0, screen_info->corners[CORNER_TOP_LEFT][state].mask, ShapeSet);
-        XShapeCombineMask (display_info->dpy,
-            MYWINDOW_XWINDOW (c->corners[CORNER_TOP_RIGHT]), ShapeBounding, 0,
-            0, screen_info->corners[CORNER_TOP_RIGHT][state].mask, ShapeSet);
 
         for (i = 0; i < BUTTON_COUNT; i++)
         {
-            my_pixmap =
-                frameGetPixmap (c, i, c->button_pressed[i] ? PRESSED : state);
-            XShapeCombineMask (display_info->dpy, MYWINDOW_XWINDOW (c->buttons[i]),
-                ShapeBounding, 0, 0, my_pixmap->mask, ShapeSet);
+            if (xfwmWindowVisible (&c->buttons[i]))
+            {
+                my_pixmap = frameGetPixmap (c, i, c->button_pressed[i] ? PRESSED : state);
+                XShapeCombineMask (display_info->dpy, MYWINDOW_XWINDOW (c->buttons[i]),
+                                   ShapeBounding, 0, 0, my_pixmap->mask, ShapeSet);
+            }
         }
 
-        if (screen_info->corners[CORNER_TOP_LEFT][state].height >
-            frameHeight (c) - frameBottom (c) + 1)
+        if (screen_info->corners[CORNER_TOP_LEFT][state].height > frameHeight (c) - frameBottom (c) + 1)
         {
-            rect.x = 0;
-            rect.y = frameHeight (c) - frameBottom (c) + 1;
-            rect.width = screen_info->corners[CORNER_TOP_LEFT][state].width;
-            rect.height =
-                screen_info->corners[CORNER_TOP_LEFT][state].height -
-                (frameHeight (c) - frameBottom (c) + 1);
-            XShapeCombineRectangles (display_info->dpy,
-                MYWINDOW_XWINDOW (c->corners[CORNER_TOP_LEFT]), ShapeBounding,
-                0, 0, &rect, 1, ShapeSubtract, 0);
+            rect.x      = 0;
+            rect.y      = frameHeight (c) - frameBottom (c) + 1;
+            rect.width  = screen_info->corners[CORNER_TOP_LEFT][state].width;
+            rect.height = screen_info->corners[CORNER_TOP_LEFT][state].height
+                           - (frameHeight (c) - frameBottom (c) + 1);
+            XShapeCombineRectangles (display_info->dpy, MYWINDOW_XWINDOW (c->corners[CORNER_TOP_LEFT]), 
+                                     ShapeBounding, 0, 0, &rect, 1, ShapeSubtract, 0);
         }
-        if (screen_info->corners[CORNER_TOP_RIGHT][state].height >
-            frameHeight (c) - frameBottom (c) + 1)
+
+        if (screen_info->corners[CORNER_TOP_RIGHT][state].height > frameHeight (c) - frameBottom (c) + 1)
         {
-            rect.x = 0;
-            rect.y = frameHeight (c) - frameBottom (c) + 1;
-            rect.width = screen_info->corners[CORNER_TOP_RIGHT][state].width;
-            rect.height =
-                screen_info->corners[CORNER_TOP_RIGHT][state].height -
-                (frameHeight (c) - frameBottom (c) + 1);
-            XShapeCombineRectangles (display_info->dpy,
-                MYWINDOW_XWINDOW (c->corners[CORNER_TOP_RIGHT]),
-                ShapeBounding, 0, 0, &rect, 1, ShapeSubtract, 0);
+            rect.x      = 0;
+            rect.y      = frameHeight (c) - frameBottom (c) + 1;
+            rect.width  = screen_info->corners[CORNER_TOP_RIGHT][state].width;
+            rect.height = screen_info->corners[CORNER_TOP_RIGHT][state].height
+                           - (frameHeight (c) - frameBottom (c) + 1);
+            XShapeCombineRectangles (display_info->dpy, MYWINDOW_XWINDOW (c->corners[CORNER_TOP_RIGHT]),
+                                     ShapeBounding, 0, 0, &rect, 1, ShapeSubtract, 0);
         }
-        if (screen_info->corners[CORNER_BOTTOM_LEFT][state].height >
-            frameHeight (c) - frameTop (c) + 1)
+
+        if (screen_info->corners[CORNER_BOTTOM_LEFT][state].height > frameHeight (c) - frameTop (c) + 1)
         {
-            rect.x = 0;
-            rect.y = 0;
-            rect.width = screen_info->corners[CORNER_BOTTOM_LEFT][state].width;
-            rect.height =
-                screen_info->corners[CORNER_BOTTOM_LEFT][state].height -
-                (frameHeight (c) - frameTop (c) + 1);
-            XShapeCombineRectangles (display_info->dpy,
-                MYWINDOW_XWINDOW (c->corners[CORNER_BOTTOM_LEFT]),
-                ShapeBounding, 0, 0, &rect, 1, ShapeSubtract, 0);
+            rect.x      = 0;
+            rect.y      = 0;
+            rect.width  = screen_info->corners[CORNER_BOTTOM_LEFT][state].width;
+            rect.height = screen_info->corners[CORNER_BOTTOM_LEFT][state].height
+                           - (frameHeight (c) - frameTop (c) + 1);
+            XShapeCombineRectangles (display_info->dpy, MYWINDOW_XWINDOW (c->corners[CORNER_BOTTOM_LEFT]),
+                                     ShapeBounding, 0, 0, &rect, 1, ShapeSubtract, 0);
         }
-        if (screen_info->corners[CORNER_BOTTOM_RIGHT][state].height >
-            frameHeight (c) - frameTop (c) + 1)
+
+        if (screen_info->corners[CORNER_BOTTOM_RIGHT][state].height > frameHeight (c) - frameTop (c) + 1)
         {
-            rect.x = 0;
-            rect.y = 0;
-            rect.width = screen_info->corners[CORNER_BOTTOM_RIGHT][state].width;
-            rect.height =
-                screen_info->corners[CORNER_BOTTOM_RIGHT][state].height -
-                (frameHeight (c) - frameTop (c) + 1);
-            XShapeCombineRectangles (display_info->dpy,
-                MYWINDOW_XWINDOW (c->corners[CORNER_BOTTOM_RIGHT]),
-                ShapeBounding, 0, 0, &rect, 1, ShapeSubtract, 0);
+            rect.x      = 0;
+            rect.y      = 0;
+            rect.width  = screen_info->corners[CORNER_BOTTOM_RIGHT][state].width;
+            rect.height = screen_info->corners[CORNER_BOTTOM_RIGHT][state].height
+                           - (frameHeight (c) - frameTop (c) + 1);
+            XShapeCombineRectangles (display_info->dpy, MYWINDOW_XWINDOW (c->corners[CORNER_BOTTOM_RIGHT]),
+                                     ShapeBounding, 0, 0, &rect, 1, ShapeSubtract, 0);
         }
 
         if (!FLAG_TEST (c->flags, CLIENT_FLAG_SHADED))
         {
-            XShapeCombineShape (display_info->dpy, temp, ShapeBounding, 0, frameTop (c),
-                MYWINDOW_XWINDOW (c->sides[SIDE_LEFT]), ShapeBounding,
-                ShapeUnion);
+            if (xfwmWindowVisible (&c->sides[SIDE_LEFT]))
+            {
+                XShapeCombineShape (display_info->dpy, temp, ShapeBounding, 0, frameTop (c),
+                                    MYWINDOW_XWINDOW (c->sides[SIDE_LEFT]), ShapeBounding, ShapeUnion);
+            }
+
+            if (xfwmWindowVisible (&c->sides[SIDE_RIGHT]))
+            {
+                XShapeCombineShape (display_info->dpy, temp, ShapeBounding, frameWidth (c) - frameRight (c), frameTop (c),
+                                    MYWINDOW_XWINDOW (c->sides[SIDE_RIGHT]), ShapeBounding, ShapeUnion);
+            }
+        }
+
+        if (xfwmWindowVisible (&c->corners[CORNER_TOP_LEFT]))
+        {
             XShapeCombineShape (display_info->dpy, temp, ShapeBounding,
-                frameWidth (c) - frameRight (c), frameTop (c),
-                MYWINDOW_XWINDOW (c->sides[SIDE_RIGHT]), ShapeBounding,
-                ShapeUnion);
-        }
-        XShapeCombineShape (display_info->dpy, temp, ShapeBounding,
-            screen_info->corners[CORNER_TOP_LEFT][state].width, 0,
-            MYWINDOW_XWINDOW (c->title), ShapeBounding, ShapeUnion);
-        XShapeCombineShape (display_info->dpy, temp, ShapeBounding,
-            screen_info->corners[CORNER_BOTTOM_LEFT][state].width,
-            frameHeight (c) - frameBottom (c),
-            MYWINDOW_XWINDOW (c->sides[SIDE_BOTTOM]), ShapeBounding,
-            ShapeUnion);
-        XShapeCombineShape (display_info->dpy, temp, ShapeBounding, 0,
-            frameHeight (c) -
-            screen_info->corners[CORNER_BOTTOM_LEFT][state].height,
-            MYWINDOW_XWINDOW (c->corners[CORNER_BOTTOM_LEFT]), ShapeBounding,
-            ShapeUnion);
-        XShapeCombineShape (display_info->dpy, temp, ShapeBounding,
-            frameWidth (c) -
-            screen_info->corners[CORNER_BOTTOM_RIGHT][state].width,
-            frameHeight (c) -
-            screen_info->corners[CORNER_BOTTOM_RIGHT][state].height,
-            MYWINDOW_XWINDOW (c->corners[CORNER_BOTTOM_RIGHT]), ShapeBounding,
-            ShapeUnion);
-        XShapeCombineShape (display_info->dpy, temp, ShapeBounding, 0, 0,
-            MYWINDOW_XWINDOW (c->corners[CORNER_TOP_LEFT]), ShapeBounding,
-            ShapeUnion);
-        XShapeCombineShape (display_info->dpy, temp, ShapeBounding,
-            frameWidth (c) - screen_info->corners[CORNER_TOP_RIGHT][state].width,
-            0, MYWINDOW_XWINDOW (c->corners[CORNER_TOP_RIGHT]), ShapeBounding,
-            ShapeUnion);
+                                screen_info->corners[CORNER_TOP_LEFT][state].width, 0,
+                                MYWINDOW_XWINDOW (c->title), ShapeBounding, ShapeUnion);
 
+            XShapeCombineShape (display_info->dpy, temp, ShapeBounding, 0, 0,
+                                MYWINDOW_XWINDOW (c->corners[CORNER_TOP_LEFT]), ShapeBounding, ShapeUnion);
+        }
+        
+        if (xfwmWindowVisible (&c->corners[CORNER_BOTTOM_LEFT]))
+        {
+            XShapeCombineShape (display_info->dpy, temp, ShapeBounding,
+                                screen_info->corners[CORNER_BOTTOM_LEFT][state].width,
+                                frameHeight (c) - frameBottom (c),
+                                MYWINDOW_XWINDOW (c->sides[SIDE_BOTTOM]), ShapeBounding, ShapeUnion);
+
+            XShapeCombineShape (display_info->dpy, temp, ShapeBounding, 0,
+                                frameHeight (c) - screen_info->corners[CORNER_BOTTOM_LEFT][state].height,
+                                MYWINDOW_XWINDOW (c->corners[CORNER_BOTTOM_LEFT]), ShapeBounding, ShapeUnion);
+        }
+        
+        if (xfwmWindowVisible (&c->corners[CORNER_BOTTOM_RIGHT]))
+        {
+            XShapeCombineShape (display_info->dpy, temp, ShapeBounding,
+                                frameWidth (c) - screen_info->corners[CORNER_BOTTOM_RIGHT][state].width,
+                                frameHeight (c) - screen_info->corners[CORNER_BOTTOM_RIGHT][state].height,
+                                MYWINDOW_XWINDOW (c->corners[CORNER_BOTTOM_RIGHT]), ShapeBounding, ShapeUnion);
+        }
+
+        if (xfwmWindowVisible (&c->corners[CORNER_TOP_RIGHT]))
+        {
+            XShapeCombineShape (display_info->dpy, temp, ShapeBounding,
+                                frameWidth (c) - screen_info->corners[CORNER_TOP_RIGHT][state].width,
+                                0, MYWINDOW_XWINDOW (c->corners[CORNER_TOP_RIGHT]), ShapeBounding, ShapeUnion);
+        }
+        
         for (i = 0; i < BUTTON_COUNT; i++)
         {
-            char b = getLetterFromButton (i, c);
-            if ((b) && strchr (screen_info->params->button_layout, b))
+            if (xfwmWindowVisible (&c->buttons[i]))
             {
                 XShapeCombineShape (display_info->dpy, temp, ShapeBounding, button_x[i],
-                    (frameTop (c) - screen_info->buttons[i][state].height) / 2,
-                    MYWINDOW_XWINDOW (c->buttons[i]), ShapeBounding,
-                    ShapeUnion);
+                                    (frameTop (c) - screen_info->buttons[i][state].height) / 2,
+                                    MYWINDOW_XWINDOW (c->buttons[i]), ShapeBounding, ShapeUnion);
             }
         }
     }
-    XShapeCombineShape (display_info->dpy, c->frame, ShapeBounding, 0, 0, temp,
-        ShapeBounding, ShapeSet);
+
+    XShapeCombineShape (display_info->dpy, c->frame, ShapeBounding, 0, 0, temp, ShapeBounding, ShapeSet);
+
     XDestroyWindow (display_info->dpy, temp);
 }
 
@@ -735,17 +761,12 @@ frameDraw (Client * c, gboolean invalidate_cache, gboolean force_shape_update)
     ScreenInfo *screen_info = NULL;
     XfwmPixmap *my_pixmap;
     gboolean requires_clearing = FALSE;
+    gboolean width_changed = FALSE;
+    gboolean height_changed = FALSE;
+    gboolean title_changed = FALSE;
     int state = ACTIVE;
-    int i;
-    int j;
-    int x;
-    int button;
-    int left;
-    int right;
-    int top_width;
-    int bottom_width;
-    int left_height;
-    int right_height;
+    int i, j, x, button, left, right;
+    int top_width, bottom_width, left_height, right_height;
     int button_x[BUTTON_COUNT];
 
     TRACE ("entering frameDraw");
@@ -808,7 +829,6 @@ frameDraw (Client * c, gboolean invalidate_cache, gboolean force_shape_update)
                 xfwmPixmapFree (&c->pm_cache.pm_sides[SIDE_BOTTOM][ACTIVE]);
                 xfwmPixmapFree (&c->pm_cache.pm_sides[SIDE_BOTTOM][INACTIVE]);
                 c->pm_cache.previous_width = c->width;
-                requires_clearing = TRUE;
             }
             if (c->pm_cache.previous_height != c->height)
             {
@@ -817,7 +837,6 @@ frameDraw (Client * c, gboolean invalidate_cache, gboolean force_shape_update)
                 xfwmPixmapFree (&c->pm_cache.pm_sides[SIDE_RIGHT][ACTIVE]);
                 xfwmPixmapFree (&c->pm_cache.pm_sides[SIDE_RIGHT][INACTIVE]);
                 c->pm_cache.previous_height = c->height;
-                requires_clearing = TRUE;
             }
         }
 
@@ -922,14 +941,14 @@ frameDraw (Client * c, gboolean invalidate_cache, gboolean force_shape_update)
         {
             frameCreateTitlePixmap (c, state, left, right,
                 &c->pm_cache.pm_title[state]);
-            requires_clearing = TRUE;
+            title_changed = TRUE;
         }
 
         if (c->pm_cache.pm_sides[SIDE_LEFT][state].pixmap == None)
         {
             xfwmPixmapCreate (screen_info, &c->pm_cache.pm_sides[SIDE_LEFT][state],
                 frameLeft (c), left_height);
-            requires_clearing = TRUE;
+            height_changed = TRUE;
         }
         fillRectangle (clientGetXDisplay (c), screen_info->screen, c->pm_cache.pm_sides[SIDE_LEFT][state].pixmap,
             screen_info->sides[SIDE_LEFT][state].pixmap, 0, 0, frameLeft (c),
@@ -942,7 +961,7 @@ frameDraw (Client * c, gboolean invalidate_cache, gboolean force_shape_update)
         {
             xfwmPixmapCreate (screen_info, &c->pm_cache.pm_sides[SIDE_RIGHT][state],
                 frameRight (c), right_height);
-            requires_clearing = TRUE;
+            height_changed = TRUE;
         }
         fillRectangle (clientGetXDisplay (c), screen_info->screen, c->pm_cache.pm_sides[SIDE_RIGHT][state].pixmap,
             screen_info->sides[SIDE_RIGHT][state].pixmap, 0, 0, frameRight (c),
@@ -955,7 +974,7 @@ frameDraw (Client * c, gboolean invalidate_cache, gboolean force_shape_update)
         {
             xfwmPixmapCreate (screen_info, &c->pm_cache.pm_sides[SIDE_BOTTOM][state],
                 bottom_width, frameBottom (c));
-            requires_clearing = TRUE;
+            width_changed = TRUE;
         }
         fillRectangle (clientGetXDisplay (c), screen_info->screen, c->pm_cache.pm_sides[SIDE_BOTTOM][state].pixmap,
             screen_info->sides[SIDE_BOTTOM][state].pixmap, 0, 0, bottom_width,
@@ -964,29 +983,45 @@ frameDraw (Client * c, gboolean invalidate_cache, gboolean force_shape_update)
             screen_info->sides[SIDE_BOTTOM][state].mask, 0, 0, bottom_width,
             frameBottom (c));
 
-        XSetWindowBackgroundPixmap (clientGetXDisplay (c), MYWINDOW_XWINDOW (c->title),
-            c->pm_cache.pm_title[state].pixmap);
-        XSetWindowBackgroundPixmap (clientGetXDisplay (c),
-            MYWINDOW_XWINDOW (c->sides[SIDE_LEFT]),
-            c->pm_cache.pm_sides[SIDE_LEFT][state].pixmap);
-        XSetWindowBackgroundPixmap (clientGetXDisplay (c),
-            MYWINDOW_XWINDOW (c->sides[SIDE_RIGHT]),
-            c->pm_cache.pm_sides[SIDE_RIGHT][state].pixmap);
-        XSetWindowBackgroundPixmap (clientGetXDisplay (c),
-            MYWINDOW_XWINDOW (c->sides[SIDE_BOTTOM]),
-            c->pm_cache.pm_sides[SIDE_BOTTOM][state].pixmap);
-        XSetWindowBackgroundPixmap (clientGetXDisplay (c),
-            MYWINDOW_XWINDOW (c->corners[CORNER_TOP_LEFT]),
-            screen_info->corners[CORNER_TOP_LEFT][state].pixmap);
-        XSetWindowBackgroundPixmap (clientGetXDisplay (c),
-            MYWINDOW_XWINDOW (c->corners[CORNER_TOP_RIGHT]),
-            screen_info->corners[CORNER_TOP_RIGHT][state].pixmap);
-        XSetWindowBackgroundPixmap (clientGetXDisplay (c),
-            MYWINDOW_XWINDOW (c->corners[CORNER_BOTTOM_LEFT]),
-            screen_info->corners[CORNER_BOTTOM_LEFT][state].pixmap);
-        XSetWindowBackgroundPixmap (clientGetXDisplay (c),
-            MYWINDOW_XWINDOW (c->corners[CORNER_BOTTOM_RIGHT]),
-            screen_info->corners[CORNER_BOTTOM_RIGHT][state].pixmap);
+        if (requires_clearing || title_changed)
+        {
+            XSetWindowBackgroundPixmap (clientGetXDisplay (c), 
+                MYWINDOW_XWINDOW (c->title),
+                c->pm_cache.pm_title[state].pixmap);
+        }
+
+        if (requires_clearing || width_changed)
+        {
+            XSetWindowBackgroundPixmap (clientGetXDisplay (c),
+                MYWINDOW_XWINDOW (c->sides[SIDE_BOTTOM]),
+                c->pm_cache.pm_sides[SIDE_BOTTOM][state].pixmap);
+        }
+
+        if (requires_clearing || height_changed)
+        {
+            XSetWindowBackgroundPixmap (clientGetXDisplay (c),
+                MYWINDOW_XWINDOW (c->sides[SIDE_LEFT]),
+                c->pm_cache.pm_sides[SIDE_LEFT][state].pixmap);
+            XSetWindowBackgroundPixmap (clientGetXDisplay (c),
+                MYWINDOW_XWINDOW (c->sides[SIDE_RIGHT]),
+                c->pm_cache.pm_sides[SIDE_RIGHT][state].pixmap);
+        }
+
+        if (requires_clearing)
+        {
+            XSetWindowBackgroundPixmap (clientGetXDisplay (c),
+                MYWINDOW_XWINDOW (c->corners[CORNER_TOP_LEFT]),
+                screen_info->corners[CORNER_TOP_LEFT][state].pixmap);
+            XSetWindowBackgroundPixmap (clientGetXDisplay (c),
+                MYWINDOW_XWINDOW (c->corners[CORNER_TOP_RIGHT]),
+                screen_info->corners[CORNER_TOP_RIGHT][state].pixmap);
+            XSetWindowBackgroundPixmap (clientGetXDisplay (c),
+                MYWINDOW_XWINDOW (c->corners[CORNER_BOTTOM_LEFT]),
+                screen_info->corners[CORNER_BOTTOM_LEFT][state].pixmap);
+            XSetWindowBackgroundPixmap (clientGetXDisplay (c),
+                MYWINDOW_XWINDOW (c->corners[CORNER_BOTTOM_RIGHT]),
+                screen_info->corners[CORNER_BOTTOM_RIGHT][state].pixmap);
+        }
 
         if (FLAG_TEST (c->flags, CLIENT_FLAG_SHADED))
         {
@@ -996,20 +1031,19 @@ frameDraw (Client * c, gboolean invalidate_cache, gboolean force_shape_update)
         else
         {
             xfwmWindowShow (&c->sides[SIDE_LEFT], 0, frameTop (c),
-                frameLeft (c), left_height, requires_clearing);
+                frameLeft (c), left_height, (requires_clearing | height_changed));
             xfwmWindowShow (&c->sides[SIDE_RIGHT],
                 frameWidth (c) - frameRight (c), frameTop (c), frameRight (c),
-                right_height, requires_clearing);
+                right_height, (requires_clearing | height_changed));
         }
 
         xfwmWindowShow (&c->title,
             screen_info->corners[CORNER_TOP_LEFT][state].width, 0, top_width,
-            frameTop (c), requires_clearing);
+            frameTop (c), (requires_clearing | title_changed));
         xfwmWindowShow (&c->sides[SIDE_BOTTOM],
             screen_info->corners[CORNER_BOTTOM_LEFT][state].width,
             frameHeight (c) - frameBottom (c), bottom_width, frameBottom (c),
-            requires_clearing);
-
+            (requires_clearing | width_changed));
         xfwmWindowShow (&c->corners[CORNER_TOP_LEFT], 0, 0,
             screen_info->corners[CORNER_TOP_LEFT][state].width,
             screen_info->corners[CORNER_TOP_LEFT][state].height,
