@@ -1,4 +1,5 @@
-/*
+/*      $Id: frame.h,v 1.15 2004/11/03 15:15:02 olivier Exp $
+ 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
         the Free Software Foundation; either version 2, or (at your option)
@@ -14,7 +15,7 @@
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2003 Olivier Fourdan
+        xfwm4    - (c) 2002-2004 Olivier Fourdan
  
  */
 
@@ -25,8 +26,13 @@
 #  include "config.h"
 #endif
 
+#include "screen.h"
 #include "client.h"
 
+int frameDecorationLeft (ScreenInfo *);
+int frameDecorationRight (ScreenInfo *);
+int frameDecorationTop (ScreenInfo *);
+int frameDecorationBottom (ScreenInfo *);
 int frameLeft (Client *);
 int frameRight (Client *);
 int frameTop (Client *);
