@@ -1,4 +1,5 @@
-/*
+/*      $Id: keyboard.h,v 1.13.2.1 2005/01/16 21:21:19 olivier Exp $
+ 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
         the Free Software Foundation; either version 2, or (at your option)
@@ -14,7 +15,7 @@
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2003 Olivier Fourdan
+        xfwm4    - (c) 2002-2004 Olivier Fourdan
  
  */
 
@@ -27,12 +28,12 @@
 
 #include <X11/keysym.h>
 
-typedef struct
+typedef struct _MyKey             MyKey;
+struct _MyKey
 {
     KeyCode keycode;
     int modifier;
-}
-MyKey;
+};
 
 extern unsigned int KeyMask;
 extern unsigned int ButtonMask;
@@ -41,7 +42,6 @@ extern unsigned int AltMask;
 extern unsigned int MetaMask;
 extern unsigned int NumLockMask;
 extern unsigned int ScrollLockMask;
-extern unsigned int CapsLockMask;
 extern unsigned int SuperMask;
 extern unsigned int HyperMask;
 
