@@ -1,4 +1,4 @@
-/*      $Id: misc.h,v 1.30 2004/10/17 08:41:54 olivier Exp $
+/*      $Id: misc.h,v 1.32 2004/12/09 22:44:20 olivier Exp $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -34,7 +34,7 @@
 void getMouseXY (ScreenInfo *, Window, int *, int *);
 Window getMouseWindow (ScreenInfo *, Window);
 GC createGC (ScreenInfo *, char *, int, XFontStruct *, int, gboolean);
-void sendClientMessage (ScreenInfo *, Window, Atom, Atom, Time);
+void sendClientMessage (ScreenInfo *, Window, int, Time);
 gboolean checkWindowOnRoot (ScreenInfo *, Window);
 void placeSidewalks(ScreenInfo *, gboolean);
 gint find_monitor_at_point (GdkScreen *screen, gint x, gint y);
