@@ -1,4 +1,5 @@
-/*
+/*      $Id: mywindow.h,v 1.12 2004/08/22 22:18:50 olivier Exp $
+ 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
         the Free Software Foundation; either version 2, or (at your option)
@@ -14,7 +15,7 @@
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2003 Olivier Fourdan
+        xfwm4    - (c) 2002-2004 Olivier Fourdan
  
  */
 
@@ -30,8 +31,8 @@
 
 #define MYWINDOW_XWINDOW(w) (w.window)
 
-typedef struct _myWindow myWindow;
-struct _myWindow
+typedef struct _xfwmWindow xfwmWindow;
+struct _xfwmWindow
 {
     Display *dpy;
     Window window;
@@ -40,9 +41,29 @@ struct _myWindow
     gboolean map;
 };
 
-void myWindowCreate (Display *, Window, myWindow *, Cursor);
-void myWindowDelete (myWindow *);
-void myWindowShow (myWindow *, int, int, int, int, gboolean);
-void myWindowHide (myWindow *);
+void xfwmWindowInit        (xfwmWindow *);
+void xfwmWindowCreate      (Display *, 
+                            Window, 
+                            xfwmWindow *, 
+                            Cursor);
+void xfwmWindowTemp        (Display *, 
+                            Window, 
+                            xfwmWindow *, 
+                            int, 
+                            int,
+                            int,
+                            int,
+                            long); 
+void xfwmWindowDelete      (xfwmWindow *);
+void xfwmWindowShow        (xfwmWindow *, 
+                            int, 
+                            int, 
+                            int, 
+                            int, 
+                            gboolean);
+void xfwmWindowHide        (xfwmWindow *);
+gboolean xfwmWindowVisible (xfwmWindow *);
+gboolean xfwmWindowDeleted (xfwmWindow *);
+
 
 #endif /* INC_MYWINDOW_H */
