@@ -1,4 +1,4 @@
-/*      $Id: netwm.c,v 1.24 2004/11/23 23:57:33 olivier Exp $
+/*      $Id: netwm.c,v 1.26.2.2 2005/02/20 14:26:49 olivier Exp $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -588,10 +588,9 @@ clientUpdateFullscreenState (Client * c)
            grab focus in focus follow mouse mode. Grab the pointer to
            avoid these effects
          */
-        XGrabPointer (clientGetXDisplay (c), c->screen_info->gnome_win, FALSE, EnterWindowMask, GrabModeAsync,
-                           GrabModeAsync, None, None, CurrentTime);
-        clientConfigure (c, &wc, CWX | CWY | CWWidth | CWHeight, NO_CFG_FLAG);
-        XUngrabPointer (clientGetXDisplay (c), CurrentTime);
+        myScreenGrabPointer (c->screen_info, EnterWindowMask, None, CurrentTime);
+        clientConfigure (c, &wc, CWX | CWY | CWWidth | CWHeight, CFG_FORCE_REDRAW);
+        myScreenUngrabPointer (c->screen_info, CurrentTime);
     }
     else
     {
@@ -970,8 +969,6 @@ clientWindowType (Client * c)
             TRACE ("atom net_wm_window_type_utility detected");
             c->type = WINDOW_UTILITY;
             c->initial_layer = WIN_LAYER_NORMAL;
-            FLAG_UNSET (c->xfwm_flags,
-                XFWM_FLAG_HAS_HIDE | XFWM_FLAG_HAS_STICK);
         }
         else if (c->type_atom == display_info->atoms[NET_WM_WINDOW_TYPE_SPLASH])
         {
@@ -990,11 +987,6 @@ clientWindowType (Client * c)
         c->type = UNSET;
         c->initial_layer = c->win_layer;
     }
-    if (FLAG_TEST (c->flags, CLIENT_FLAG_STATE_MODAL))
-    {
-        TRACE ("window is modal");
-        c->type = WINDOW_MODAL_DIALOG;
-    }
 
     if (clientIsValidTransientOrModal (c))
     {
