@@ -1,4 +1,4 @@
-/*      $Id: settings.c,v 1.121 2004/12/05 23:02:28 olivier Exp $
+/*      $Id: settings.c,v 1.122.2.1 2005/02/04 21:25:44 olivier Exp $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -408,7 +408,7 @@ loadRcData (ScreenInfo *screen_info, Settings rc[])
 
     if (!parseRc ("defaults", PACKAGE_DATADIR, rc))
     {
-        g_warning ("%s: Missing defaults file", g_get_prgname ());
+        g_warning ("Missing defaults file");
         exit (1);
     }
     keythemevalue = getValue ("keytheme", rc);
@@ -637,8 +637,8 @@ loadTheme (ScreenInfo *screen_info, Settings rc[])
 
     rc[0].value  = get_style (widget, "fg",    "selected");
     rc[1].value  = get_style (widget, "fg",    "insensitive");
-    rc[2].value  = get_style (widget, "fg",    "selected");
-    rc[3].value  = get_style (widget, "fg",    "insensitive");
+    rc[2].value  = get_style (widget, "fg",    "normal");
+    rc[3].value  = get_style (widget, "fg",    "normal");
     rc[4].value  = get_style (widget, "bg",    "selected");
     rc[5].value  = get_style (widget, "light", "selected");
     rc[6].value  = get_style (widget, "dark",  "selected");
@@ -961,14 +961,13 @@ loadKeyBindings (ScreenInfo *screen_info, Settings rc[])
         keytheme = getThemeDir (keythemevalue, KEYTHEMERC);
         if (!parseRc (KEYTHEMERC, keytheme, rc))
         {
-            g_warning ("%s: specified key theme \"%s\" missing, using default",
-                       g_get_prgname (), keythemevalue);
+            g_warning ("Specified key theme \"%s\" missing, using default", keythemevalue);
         }
         g_free (keytheme);
 
         if (!checkRc (rc))
         {
-            g_warning ("%s: Missing values in defaults file", g_get_prgname ());
+            g_warning ("Missing values in defaults file");
             return FALSE;
         }
     }
