@@ -1,4 +1,5 @@
-/*
+/*      $Id: startup_notification.h,v 1.13 2004/08/22 22:18:50 olivier Exp $
+ 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
         the Free Software Foundation; either version 2, or (at your option)
@@ -14,7 +15,7 @@
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
         Metacity - (c) 2003 Havoc Pennington
-        xfwm4    - (c) 2003 Olivier Fourdan
+        xfwm4    - (c) 2002-2004 Olivier Fourdan
  
  */
 
@@ -30,17 +31,18 @@
 
 #include <libsn/sn.h>
 
+#include "screen.h"
 #include "client.h"
 
 void sn_client_startup_properties (Client *);
-void sn_init_display (Display *, int);
-void sn_close_display (void);
-void sn_process_event (XEvent * event);
+void sn_init_display              (ScreenInfo *);
+void sn_close_display             (void);
+void sn_process_event             (XEvent *);
 
 #else /* HAVE_LIBSTARTUP_NOTIFICATION */
 
 #define sn_client_startup_properties(c) ;
-#define sn_init_display(d,i) ;
+#define sn_init_display(d) ;
 #define sn_close_display() ;
 #define sn_process_event(e) ;
 
